﻿using System;
using System.Runtime.InteropServices;

namespace WpfAudioPlayer.Models.Wmf
{
	/// <summary>
	/// Helper class who encapsulates INSSBuffer buffers.
	/// </summary>
	class WmfNSSBuffer
	{
		private INSSBuffer _buffer;
		private uint _length;
		private uint _maxLength;
		private IntPtr _bufferPtr;
		private uint _position = 0;

		/// <summary>
		/// NSSBuffer constructor
		/// </summary>
		/// <param name="buff">INSSBuffer to wrap</param>
		public WmfNSSBuffer( INSSBuffer buff )
		{
			_buffer = buff;
			_buffer.GetBufferAndLength( out _bufferPtr, out _length );
			_buffer.GetMaxLength( out _maxLength );
		}

		/// <summary>
		/// Length of the buffer. Wraps INSSBuffer.GetLength and INSSBuffer.SetLength
		/// </summary>
		public uint Length
		{
			get
			{
				return _length;
			}
			set
			{
				if( value <= _maxLength )
				{
					_buffer.SetLength( value );
					_length = value;
					if( _position > _length )
					{
						_position = _length;
					}
				}
				else
				{
					throw new ArgumentOutOfRangeException();
				}
			}
		}

		/// <summary>
		/// Read/Write the position for Read or Write operations
		/// </summary>
		public uint Position
		{
			get
			{
				return _position;
			}
			set
			{
				if( value <= _length )
				{
					_position = value;
				}
				else
				{
					throw new ArgumentOutOfRangeException();
				}
			}
		}

		/// <summary>
		/// Reference to the internal INSSBuffer
		/// </summary>
		public INSSBuffer Buffer
		{
			get { return _buffer; }
		}

		/// <summary>
		/// Reads from the wrapped buffer to a byte array. 
		/// Position is increased by the number of bytes read.
		/// </summary>
		/// <param name="buffer">Destination byte array</param>
		/// <param name="offset">Position in the destination array where to start copying</param>
		/// <param name="count">Number of bytes to read</param>
		/// <returns>Number of bytes read. Zero means than Position was at buffer length.</returns>
		public int Read( byte[] buffer, int offset, int count )
		{
			if( _position < _length )
			{
				IntPtr src = ( IntPtr )( _bufferPtr.ToInt32() + _position );
				int ToCopy = Math.Min( count, ( int )( this.Length - this.Position ) );
				Marshal.Copy( src, buffer, offset, ToCopy );
				_position += ( uint )ToCopy;
				return ToCopy;
			}
			else
			{
				return 0;
			}
		}

		/// <summary>
		/// Write to the wrapped buffer from a byte array. 
		/// Position is increased by the number of byte written
		/// </summary>
		/// <param name="buffer">Source byte array</param>
		/// <param name="offset">Index from where start copying</param>
		/// <param name="count">Number of bytes to copy</param>
		public void Write( byte[] buffer, int offset, int count )
		{
			if( buffer == null )
			{
				throw new ArgumentNullException( "buffer" );
			}
			if( offset < 0 )
			{
				throw new ArgumentOutOfRangeException( "offset" );
			}
			if( ( count <= 0 ) || ( ( _position + count ) > _length ) )
			{
				throw new ArgumentOutOfRangeException( "count" );
			}
			IntPtr dest = ( IntPtr )( _bufferPtr.ToInt32() + _position );
			Marshal.Copy( buffer, offset, dest, count );
			_position += ( uint )count;
		}
	}
}
